? ip_doopts.diff
Index: ip_fastfwd.c
===================================================================
RCS file: /home/ncvs/src/sys/netinet/ip_fastfwd.c,v
retrieving revision 1.10
diff -u -p -r1.10 ip_fastfwd.c
--- ip_fastfwd.c	3 May 2004 16:07:13 -0000	1.10
+++ ip_fastfwd.c	5 May 2004 01:43:51 -0000
@@ -292,8 +292,16 @@ ip_fastforward(struct mbuf *m)
 	/*
 	 * Only IP packets without options
 	 */
-	if (ip->ip_hl != (sizeof(struct ip) >> 2))
-		return 0;
+	if (ip->ip_hl != (sizeof(struct ip) >> 2)) {
+		if (ip_doopts == 1)
+			return 0;
+		else if (ip_doopts == 2) {
+			icmp_error(m, ICMP_UNREACH, ICMP_UNREACH_FILTER_PROHIB,
+				0, NULL);
+			return 1;
+		}
+		/* else ignore IP options and continue */
+	}
 
 	/*
 	 * Only unicast IP, not from loopback, no L2 or IP broadcast,
Index: ip_input.c
===================================================================
RCS file: /home/ncvs/src/sys/netinet/ip_input.c,v
retrieving revision 1.270
diff -u -p -r1.270 ip_input.c
--- ip_input.c	2 May 2004 15:10:16 -0000	1.270
+++ ip_input.c	5 May 2004 01:43:52 -0000
@@ -110,6 +110,10 @@ SYSCTL_INT(_net_inet_ip, IPCTL_ACCEPTSOU
     CTLFLAG_RW, &ip_acceptsourceroute, 0, 
     "Enable accepting source routed IP packets");
 
+int		ip_doopts = 1;	/* 0 = ignore, 1 = process, 2 = reject */
+SYSCTL_INT(_net_inet_ip, OID_AUTO, dooptions, CTLFLAG_RW,
+    &ip_doopts, 0, "Enable IP options processing ([LS]SRR, RR, TS)");
+
 static int	ip_keepfaith = 0;
 SYSCTL_INT(_net_inet_ip, IPCTL_KEEPFAITH, keepfaith, CTLFLAG_RW,
 	&ip_keepfaith,	0,
@@ -509,7 +513,8 @@ pass:
 	 * to be sent and the original packet to be freed).
 	 */
 	ip_nhops = 0;		/* for source routed packets */
-	if (hlen > sizeof (struct ip) && ip_dooptions(m, 0, args.next_hop))
+	if (hlen > sizeof (struct ip) && ip_doopts &&
+	    ip_dooptions(m, 0, args.next_hop))
 		return;
 
         /* greedy RSVP, snatches any PATH packet of the RSVP protocol and no
@@ -704,7 +709,7 @@ ours:
 	 * if the packet is destined for us.
 	 */
 	if (ipstealth && hlen > sizeof (struct ip) &&
-	    ip_dooptions(m, 1, args.next_hop))
+	    ip_doopts && ip_dooptions(m, 1, args.next_hop))
 		return;
 #endif /* IPSTEALTH */
 
@@ -1286,6 +1291,13 @@ ip_dooptions(struct mbuf *m, int pass, s
 	struct in_addr *sin, dst;
 	n_time ntime;
 	struct	sockaddr_in ipaddr = { sizeof(ipaddr), AF_INET };
+
+	/* reject packets with IP options */
+	if (ip_doopts == 2) {
+		type = ICMP_UNREACH;
+		code = ICMP_UNREACH_FILTER_PROHIB;
+		goto bad;
+	}
 
 	dst = ip->ip_dst;
 	cp = (u_char *)(ip + 1);
Index: ip_var.h
===================================================================
RCS file: /home/ncvs/src/sys/netinet/ip_var.h,v
retrieving revision 1.86
diff -u -p -r1.86 ip_var.h
--- ip_var.h	2 May 2004 06:36:30 -0000	1.86
+++ ip_var.h	5 May 2004 01:43:52 -0000
@@ -147,6 +147,7 @@ extern u_short	ip_id;				/* ip packet ct
 #endif
 extern int	ip_defttl;			/* default IP ttl */
 extern int	ipforwarding;			/* ip forwarding */
+extern int	ip_doopts;			/* process or ignore IP options */
 #ifdef IPSTEALTH
 extern int	ipstealth;			/* stealth forwarding */
 #endif
